/* 
 *  file:        scroll.c 
 *  description: a multi-directionnal scrolling demo
 *  author:      David Michel (dmichel@easynet.fr)
 *  date:        2001.02.19
 *  files:       scroll.c scroll.pcx scroll.bmp scroll.fmp
 *  version:     1.0
 */

#include "huc.h"

/*  scroll-area size */
#define SCR_W    17
#define SCR_H    14
/*  screen size */
#define BAT_W    32
#define BAT_H    16
/*  map size */
#define MAP_W    32
#define MAP_H    32
/*  nb tile */
#define NB_TILE  12

/*  vram allocation */
#define TILE_VRAM    0x1000
#define PANEL_VRAM   0x4000
#define SPRITE_VRAM  0x7000

/*  palette allocation */
#define TILE_PAL     4
#define PANEL_PAL    2
#define SPRITE_PAL  16

/*  map */
#incbin(game_map,  "scroll.fmp")

/*  tiles */
#inctile(tile_gfx, "scroll.pcx", 4, 3)
#incpal(tile_pal,  "scroll.pcx", 0)

/*  control panel */
#incchr(panel_gfx, "scroll.pcx", 0, 48, 32, 4)
#incpal(panel_pal, "scroll.pcx", 2)
#incbat(panel_bat, "scroll.pcx", PANEL_VRAM, 0, 48, 32, 4)

/*  palette index for tiles (one index per tile) */
const char tile_pal_ref[NB_TILE] = {
	0x40, 0x40, 0x40, 0x40,
	0x40, 0x40, 0x40, 0x40,
	0x40, 0x40, 0x40, 0x40
};

/*  scroll vars */
int p_x, p_y;
int s_x, s_y;
int t_x, t_y;
int m_x, m_y;


main()
{
	int dx, dy;
	int tx, ty;

	/*  init */
	init_map(0, 0);

	/*  sync */
	vsync();

	/*  main loop */
	for (;;)
	{
		/*  check gamepad */
		dx = 0;
		dy = 0;

		if (joy(0) & JOY_UP)
			dy = -1;
		if (joy(0) & JOY_DOWN)
			dy =  1;
		if (joy(0) & JOY_LEFT)
			dx = -1;
		if (joy(0) & JOY_RGHT)
			dx =  1;

		/*  move */
		tx = p_x + dx;
		ty = p_y + dy;

		if (tx < 0)
			dx = 0;
		if (tx > ((MAP_W - SCR_W + 1) << 4))
			dx = 0;
		if (ty < 0)
			dy = 0;
		if (ty > ((MAP_H - SCR_H + 2) << 4))
			dy = 0;

		/*  scroll */
		p_x += dx;
		p_y += dy;
		move(dx, dy);
		vsync();
	}
}


/* ----
 * init_map()
 * ----
 *
 */
init_map(x, y)
int x;
int y;
{
	/*  disable display */
	disp_off();
	vsync();

	/*  clear display */
	cls();

	/*  init map */
	set_map_data(game_map, MAP_W, MAP_H);
	set_tile_data(tile_gfx, NB_TILE, tile_pal_ref);
	load_tile(TILE_VRAM);
	load_palette(TILE_PAL, tile_pal, 1);
	load_map(0, 0, x, y, SCR_W, SCR_H);

	/*  init coord. */
	p_x = (x >> 4);
	p_y = (y >> 4);
	s_x = 0;
	s_y = 0;
	t_x = 0;
	t_y = 0;
	m_x = x;
	m_y = y;

	/*  init font */
	set_font_color(1, 10);
	set_font_pal(PANEL_PAL);
	load_default_font();

	/*  init control panel */
	load_vram(PANEL_VRAM, panel_gfx, 2048);
	load_bat(0x0700, panel_bat, 32, 4);
	load_palette(PANEL_PAL, panel_pal, 1);

	/*  split screen */
	scroll(0, 0,   0,   0, 191, 0xC0);
	scroll(2, 0, 224, 192, 223, 0x80);

	/*  enable display */
	disp_on();
	vsync();
}


/* ----
 * move()
 * ----
 * scrolling
 *
 */
move(dx, dy)
int dx;
int dy;
{
	int  x, y;
	char h;

	/* horizontal move */
	if (dx) {
		s_x += dx;

		/* left */
		if (s_x < 0) {
			s_x &= 0x0F;
			t_x -= 1;
			m_x -= 1;

			if (t_x  < 0)
				t_x += BAT_W;
			if (m_x  < 0)
				m_x += MAP_W;

			load_column(t_x, t_y, m_x, m_y);
		}
		/* right */
		else if (s_x > 15) {
			s_x &= 0x0F;
			t_x += 1;
			m_x += 1;

			if (t_x >= BAT_W)
				t_x -= BAT_W;
			if (m_x >= MAP_W)
				m_x -= MAP_W;

			load_column(t_x + 16, t_y, m_x + 16, m_y);
		}
	}

	/* vertical move */
	if (dy) {
		s_y += dy;

		/* up */
		if (s_y < 0) {
			s_y &= 0x0F;
			t_y -= 1;
			m_y -= 1;

			if (t_y  < 0)
				t_y += SCR_H;
			if (m_y  < 0)
				m_y += MAP_H;

			load_row(t_x, t_y, m_x, m_y);
		}
		/* down */
		else if (s_y > 15) {
			s_y &= 0x0F;
			t_y += 1;
			m_y += 1;

			if (t_y >= SCR_H)
				t_y -= SCR_H;
			if (m_y >= MAP_H)
				m_y -= MAP_H;

			load_row(t_x, t_y + 12, m_x, m_y + 12);
		}
	}

	/* scroll */
	x = (t_x << 4) + s_x;
	y = (t_y << 4) + s_y;

	if (y <= 32) {
		/* one area */
		scroll(0, x, y,   0, 191, 0xC0);
		scroll(1, 0, 0, 255, 255, 0x00);
	}
	else {
		/* splitted */
		h = (SCR_H << 4) - y;
		scroll(0, x, y, 0, h - 1, 0xC0);
		scroll(1, x, 0, h,   191, 0xC0);
	}
}

load_column(x, y, mx, my)
char x;
char y;
int  mx;
int  my;
{
	char h;

	if (y) {
		/* splitted copy */
		h = SCR_H - y;
		load_map(x, y, mx, my,     1, h);
		load_map(x, 0, mx, my + h, 1, SCR_H - h);
	}
	else {
		/* one block copy */
		load_map(x, y, mx, my, 1, SCR_H);
	}
}

load_row(x, y, mx, my)
char x;
char y;
int  mx;
int  my;
{
	char w;

	if (y >= SCR_H)
		y -= SCR_H;

	if ((x + SCR_W) > BAT_W) {
		/* splitted copy */
		w = BAT_W - x;
		load_map(x, y, mx,     my, w,         1);
		load_map(0, y, mx + w, my, SCR_W - w, 1);
	}
	else {
		/* one block copy */
		load_map(x, y, mx, my, SCR_W, 1);
	}
}

